import ctypes
import os


class IcpdasEcatLib:
    def __init__(self):
        if os.name == "nt":
            libecatdevice = ctypes.cdll.LoadLibrary("/icpdas/Ecat-M801/API/x64/libecatdevice.dll")
        else:
            libecatdevice = ctypes.cdll.LoadLibrary("/opt/icpdas/ecat_m801/lib/libecatdevice.so")

        self.ECAT_GetDeviceCnt = libecatdevice.ECAT_GetDeviceCnt
        self.ECAT_GetDeviceCnt.argtypes = [ctypes.POINTER(ctypes.c_uint16), ctypes.POINTER(ctypes.c_uint8)]
        self.ECAT_GetDeviceCnt.restype = ctypes.c_int32

        self.ECAT_OpenDevice = libecatdevice.ECAT_OpenDevice
        self.ECAT_OpenDevice.argtypes = [ctypes.c_uint16]
        self.ECAT_OpenDevice.restype = ctypes.c_int32

        self.ECAT_CloseDevice = libecatdevice.ECAT_CloseDevice
        self.ECAT_CloseDevice.argtypes = [ctypes.c_ushort]
        self.ECAT_CloseDevice.restype = ctypes.c_int32

        self.ECAT_GetDeviceState = libecatdevice.ECAT_GetDeviceState
        self.ECAT_GetDeviceState.argtypes = [ctypes.c_uint16, ctypes.POINTER(ctypes.c_uint32),
                                             ctypes.POINTER(ctypes.c_uint32), ctypes.POINTER(ctypes.c_uint32),
                                             ctypes.POINTER(ctypes.c_uint32)]
        self.ECAT_GetDeviceState.restype = ctypes.c_int32

        self.ECAT_StopDeviceOpTask = libecatdevice.ECAT_StopDeviceOpTask
        self.ECAT_StopDeviceOpTask.argtypes = [ctypes.c_uint16]
        self.ECAT_StopDeviceOpTask.restype = ctypes.c_int32

        self.ECAT_UploadFirmware = libecatdevice.ECAT_UploadFirmware
        self.ECAT_UploadFirmware.argtypes = [ctypes.c_uint16, ctypes.c_uint32,
                                             ctypes.POINTER(ctypes.c_uint8), ctypes.c_uint32, ctypes.c_uint8]
        self.ECAT_UploadFirmware.restype = ctypes.c_int32

        self.ECAT_GetFirmwareVersion = libecatdevice.ECAT_GetFirmwareVersion
        self.ECAT_GetFirmwareVersion.argtypes = [ctypes.c_uint16, ctypes.c_char_p, ctypes.POINTER(ctypes.c_uint16)]
        self.ECAT_GetFirmwareVersion.restype = ctypes.c_int32

        self.ECAT_GetProcessTime = libecatdevice.ECAT_GetProcessTime
        self.ECAT_GetProcessTime.argtypes = [ctypes.c_uint16, ctypes.POINTER(ctypes.c_double)]
        self.ECAT_GetProcessTime.restype = ctypes.c_int32

        self.ECAT_UploadEc_Generic = libecatdevice.ECAT_UploadEc_Generic
        self.ECAT_UploadEc_Generic.argtypes = [ctypes.c_uint16, ctypes.c_uint32,
                                               ctypes.POINTER(ctypes.c_uint8), ctypes.c_uint32, ctypes.c_uint8]
        self.ECAT_UploadEc_Generic.restype = ctypes.c_int32

        self.ECAT_UploadEc_Master = libecatdevice.ECAT_UploadEc_Master
        self.ECAT_UploadEc_Master.argtypes = [ctypes.c_uint16, ctypes.c_uint32,
                                              ctypes.POINTER(ctypes.c_uint8), ctypes.c_uint32, ctypes.c_uint8]
        self.ECAT_UploadEc_Master.restype = ctypes.c_int32

        self.ECAT_GetIghVir = libecatdevice.ECAT_GetIghVir
        self.ECAT_GetIghVir.argtypes = [ctypes.c_uint16, ctypes.POINTER(ctypes.c_uint16)]
        self.ECAT_GetIghVir.restype = ctypes.c_int32

        self.ECAT_SetIghVir = libecatdevice.ECAT_SetIghVir
        self.ECAT_SetIghVir.argtypes = [ctypes.c_uint16, ctypes.c_uint16]
        self.ECAT_SetIghVir.restype = ctypes.c_int32

        self.ECAT_Upload_Ixecat = libecatdevice.ECAT_Upload_Ixecat
        self.ECAT_Upload_Ixecat.argtypes = [ctypes.c_uint16, ctypes.c_uint32,
                                            ctypes.POINTER(ctypes.c_uint8), ctypes.c_uint32, ctypes.c_uint8]
        self.ECAT_Upload_Ixecat.restype = ctypes.c_int32

        self.ECAT_Upload_Ixpciecat = libecatdevice.ECAT_Upload_Ixpciecat
        self.ECAT_Upload_Ixpciecat.argtypes = [ctypes.c_uint16, ctypes.c_uint32,
                                               ctypes.POINTER(ctypes.c_uint8), ctypes.c_uint32, ctypes.c_uint8]
        self.ECAT_Upload_Ixpciecat.restype = ctypes.c_int32

        self.ECAT_McGetIxecatVir = libecatdevice.ECAT_McGetIxecatVir
        self.ECAT_McGetIxecatVir.argtypes = [ctypes.c_uint16, ctypes.POINTER(ctypes.c_uint16)]
        self.ECAT_McGetIxecatVir.restype = ctypes.c_int32

        self.ECAT_McSetIxecatVir = libecatdevice.ECAT_McSetIxecatVir
        self.ECAT_McSetIxecatVir.argtypes = [ctypes.c_uint16, ctypes.c_uint16]
        self.ECAT_McSetIxecatVir.restype = ctypes.c_int32

        self.ECAT_GetDeviceXilinxVer = libecatdevice.ECAT_GetDeviceXilinxVer
        self.ECAT_GetDeviceXilinxVer.argtypes = [ctypes.c_uint16, ctypes.POINTER(ctypes.c_uint16)]
        self.ECAT_GetDeviceXilinxVer.restype = ctypes.c_int32

        self.ECAT_GetDeviceXilinx_Process = libecatdevice.ECAT_GetDeviceXilinx_Process
        self.ECAT_GetDeviceXilinx_Process.argtypes = [ctypes.c_uint16, ctypes.POINTER(ctypes.c_uint8)]
        self.ECAT_GetDeviceXilinx_Process.restype = ctypes.c_int32

        self.ECAT_WriteDeviceXilinx = libecatdevice.ECAT_WriteDeviceXilinx
        self.ECAT_WriteDeviceXilinx.argtypes = [ctypes.c_uint16, ctypes.c_char_p]
        self.ECAT_WriteDeviceXilinx.restype = ctypes.c_int32

        if os.name == "nt":
            self.ECAT_OpenDeviceForFlash = libecatdevice.ECAT_OpenDeviceForFlash
            self.ECAT_OpenDeviceForFlash.argtypes = [ctypes.c_uint16, ctypes.c_uint8]
            self.ECAT_OpenDeviceForFlash.restype = ctypes.c_int32

            self.ECAT_CloseDeviceForFlash = libecatdevice.ECAT_CloseDeviceForFlash
            self.ECAT_CloseDeviceForFlash.argtypes = [ctypes.c_uint16]
            self.ECAT_CloseDeviceForFlash.restype = ctypes.c_int32

