import os
import time
import ctypes
import threading
from ecatLib import IcpdasEcatLib

bar_len = 20
TimeOut = 5
fill_symbol = "#"
default_symbol = "-"
last_progress = 0
DeviceNo = 0
# time.sleep(0.03)

up_fpga_ver = 0
up_fpga_file = ""
cur_fpga_ver = 0

up_daemon_ver = 0
up_daemon_sub_ver = 0
up_daemon_file = ""
cur_daemon_ver = 0
cur_daemon_sub_ver = 0

up_ixe_ver1 = 0
up_ixe_ver2 = 0
up_ixe_file1 = ""
up_ixe_file2 = ""
cur_ixe_ver = 0

up_igh_ver1 = 0
up_igh_ver2 = 0
up_igh_file1 = ""
up_igh_file2 = ""
cur_igh_ver = 0


def int_(str_):
    if str_.find(".") != -1:
        str_ = str_[:str_.find(".")]
    return int(str_)


def updateProgress(progress_):  # 0 ~ 1
    progress__ = int(progress_ * 100)
    progress_ = int(progress_ * bar_len)
    global last_progress
    if last_progress != progress__:
        if os.name == 'nt':
            os.system('cls')
        else:
            os.system('clear')
        print("progress: [" + progress_ * fill_symbol + (bar_len - progress_) * default_symbol + "]" + str(int(progress__)) + "%")
        last_progress = progress__


def InitDevice():
    DeviceCnt = ctypes.c_uint16()
    CardId = (ctypes.c_uint8 * 16)()
    ret = elib.ECAT_GetDeviceCnt(ctypes.byref(DeviceCnt), CardId)
    if ret != 0 or DeviceCnt.value == 0:
        Mesg = ""
        if ret == 0 and DeviceCnt.value == 0:
            Mesg = "Card Not Found"
        else:
            Mesg = "Card Not Found, Error{}".format(ret)
        print(Mesg)
        return -1
    else:
        global DeviceNo
        DeviceNo = CardId[0]
    return 0


def GetFpgaFileVer():
    found_ = -1
    chose_ver = 0
    chose_sub_ver = 0
    chose_file = ""
    files = os.listdir("./firmware")
    for FileName in files:
        if FileName.find("xilinx") == -1:
            continue  # next File
        ver = FileName.replace("xilinx_V", "", 1)
        ver = ver.replace("xilinx_v", "", 1)
        ver = int_(ver)
        if ver >= chose_ver:
            chose_ver = ver
            chose_file = FileName
    global up_fpga_ver
    global up_fpga_file

    up_fpga_ver = chose_ver
    up_fpga_file = chose_file


def GetFpgaCurVer():
    if os.name == 'nt':
        ret = elib.ECAT_OpenDeviceForFlash(DeviceNo, 0)
    else:
        ret = elib.ECAT_OpenDevice(DeviceNo)
    if ret != 0:
        print("Failed to open EcatDevice, {}".format(ret))
        return -1
    ver = ctypes.c_uint16(0)
    retry_ = 5
    while retry_:
        retry_ = retry_ - 1
        ret = elib.ECAT_GetDeviceXilinxVer(DeviceNo, ctypes.byref(ver))
        if ret == 0:
            break
    if ret != 0:
        print("Failed to Get Firmware version, {}".format(ret))
        if os.name == 'nt':
            elib.ECAT_CloseDeviceForFlash(DeviceNo)
        else:
            elib.ECAT_CloseDevice(DeviceNo)
        return -1

    if os.name == 'nt':
        elib.ECAT_CloseDeviceForFlash(DeviceNo)
    else:
        elib.ECAT_CloseDevice(DeviceNo)

    ver = ver.value

    global cur_fpga_ver
    cur_fpga_ver = ver
    return 0


def Upload_Xilinx_Thread():
    file_name = "./firmware/" + up_fpga_file
    ret = elib.ECAT_WriteDeviceXilinx(DeviceNo, file_name.encode('utf-8'))
    print("xilinxDone.{}".format(ret))


def UpdateFpga(retry_):
    update = 0
    global cur_fpga_ver
    if up_fpga_ver > cur_fpga_ver:
        update = 1
        cur_fpga_ver = up_fpga_ver
    if update == 0:
        return 0
    if os.name == 'nt':
        ret = elib.ECAT_OpenDeviceForFlash(DeviceNo, 0)
    else:
        ret = elib.ECAT_OpenDevice(DeviceNo)
    if ret != 0:
        print("Failed to open EcatDevice, {}".format(ret))
        return -1
    t = threading.Thread(target=Upload_Xilinx_Thread)
    t.start()
    progess = ctypes.c_uint8(0)
    while 1:
        elib.ECAT_GetDeviceXilinx_Process(DeviceNo, ctypes.byref(progess))
        updateProgress(progess.value / 100)
        if progess.value == 100:
            t.join()
            if os.name == 'nt':
                elib.ECAT_CloseDeviceForFlash(DeviceNo)
            else:
                elib.ECAT_CloseDevice(DeviceNo)
            return 0


def GetDaemonFileVer():
    found_ = -1
    chose_ver = 0
    chose_sub_ver = 0
    chose_file = ""
    files = os.listdir("./firmware")
    for FileName in files:
        if FileName.find("EcatDevDaemon") == -1:
            continue  # next File
        ver = FileName.replace("EcatDevDaemon_V1.0.", "", 1)  # ex:25-5 or 26
        ver = ver.replace("EcatDevDaemon_v1.0.", "", 1)  # ex:25-5 or 26
        if ver.find("-") != -1:  # has sub version
            sub_ver = ver[ver.find("-") + 1:]
            ver = ver[:ver.find("-")]
        else:
            sub_ver = "0"
        ver = int(ver)
        sub_ver = int(sub_ver)
        if ver > chose_ver:
            chose_ver = ver
            chose_sub_ver = sub_ver
            chose_file = FileName
        if ver == chose_ver and sub_ver > chose_sub_ver:
            chose_sub_ver = sub_ver
            chose_file = FileName
    global up_daemon_ver
    global up_daemon_sub_ver
    global up_daemon_file

    up_daemon_ver = chose_ver
    up_daemon_sub_ver = chose_sub_ver
    up_daemon_file = chose_file


def GetDaemonCurVer():
    ret = elib.ECAT_OpenDevice(DeviceNo)
    if ret != 0:
        print("Failed to open EcatDevice, {}".format(ret))
        return -1
    size = ctypes.c_uint16()
    buff = ctypes.create_string_buffer(255)
    ret = elib.ECAT_GetFirmwareVersion(DeviceNo, buff, ctypes.byref(size))
    if ret != 0:
        print("Failed to Get Firmware version, {}".format(ret))
        return -1
    str_ = buff.value.decode()
    v1 = str_[0:str_.find(".")]
    str_ = str_[str_.find(".") + 1:]
    v2 = str_[0:str_.find(".")]
    str_ = str_[str_.find(".") + 1:]
    ver = str_[0:str_.find(".")]

    if ver.find("-") != -1:  # has sub version
        sub_ver = ver[ver.find("-") + 1:]
        ver = ver[:ver.find("-")]
    else:
        sub_ver = "0"
    ver = int(ver)
    sub_ver = int(sub_ver)

    global cur_daemon_ver
    global cur_daemon_sub_ver

    cur_daemon_ver = ver
    cur_daemon_sub_ver = sub_ver

    elib.ECAT_StopDeviceOpTask(DeviceNo)
    link_up = ctypes.c_uint32()
    slave_resp = ctypes.c_uint32()
    al_states = ctypes.c_uint32()
    wc = ctypes.c_uint32()
    statTime_ = time.time()
    while 1:
        passTime_ = time.time() - statTime_
        ret = elib.ECAT_GetDeviceState(DeviceNo, ctypes.byref(link_up), ctypes.byref(slave_resp),
                                       ctypes.byref(al_states), ctypes.byref(wc))
        if ret == 0 and (al_states.value == 2 or al_states.value == 0):
            return 0
        if passTime_ > 10:
            print("Please unplug Ethernet cable attached on the Master Card and try again")
            return -1
        time.sleep(1)


def UpdateDaemon(retry_):
    update = 0
    global cur_daemon_ver
    global cur_daemon_sub_ver

    if up_daemon_ver > cur_daemon_ver:
        update = 1
        cur_daemon_ver = up_daemon_ver
        cur_daemon_sub_ver = up_daemon_sub_ver
    if up_daemon_ver == cur_daemon_ver and up_daemon_sub_ver > cur_daemon_sub_ver:
        update = 1
        cur_daemon_ver = up_daemon_ver
        cur_daemon_sub_ver = up_daemon_sub_ver
    if update == 0:
        return 0

    file_name = "./firmware/" + up_daemon_file
    file_size = os.path.getsize(file_name)
    ret = -1
    while retry_:
        retry_ = retry_ - 1
        upload_size = 0
        last_flag = 0
        with open(file_name, mode='rb') as file:
            while 1:
                file_data = file.read(256)
                if len(file_data) == 0:
                    updateProgress(1)
                    file.close()
                    break
                offset = ctypes.c_uint32(upload_size)
                data = (ctypes.c_uint8 * len(file_data)).from_buffer(bytearray(file_data))
                data_size = ctypes.c_uint32(len(file_data))
                if file_size == (upload_size + len(file_data)):
                    last_flag = ctypes.c_uint8(1)
                else:
                    last_flag = ctypes.c_uint8(0)
                ret = elib.ECAT_UploadFirmware(DeviceNo, offset, data, data_size, last_flag)
                if ret != 0:
                    file.close()
                    break
                upload_size += len(file_data)
                updateProgress(upload_size / file_size)
        if ret == 0:
            break
        time.sleep(1)
    if ret != 0:
        print("update firmware failed, {}".format(ret))
        return ret

    elib.ECAT_CloseDevice(DeviceNo)

    statTime_ = time.time()
    while 1:
        passTime_ = time.time() - statTime_
        ret = elib.ECAT_OpenDevice(DeviceNo)
        if ret == 0:
            updateProgress(1)
            # print("update success")
            return 0
        updateProgress(passTime_ / 15)
        if passTime_ > 15:
            print("Time out\nPlease power off/on the computer and try again\n")
            return -1
        time.sleep(1)


def GetIxeFileVer1():
    found_ = -1
    chose_ver = 0
    chose_sub_ver = 0
    chose_file = ""
    files = os.listdir("./firmware")
    for FileName in files:
        if FileName.find("ixecat") == -1:
            continue  # next File
        ver = FileName.replace("ixecat_V", "", 1)
        ver = ver.replace("ixecat_v", "", 1)
        ver = int_(ver)
        if ver > chose_ver:
            chose_ver = ver
            chose_file = FileName
    global up_ixe_ver1
    global up_ixe_file1

    up_ixe_ver1 = chose_ver
    up_ixe_file1 = chose_file


def GetIxeFileVer2():
    found_ = -1
    chose_ver = 0
    chose_sub_ver = 0
    chose_file = ""
    files = os.listdir("./firmware")
    for FileName in files:
        if FileName.find("ixpciecat") == -1:
            continue  # next File
        ver = FileName.replace("ixpciecat_V", "", 1)
        ver = ver.replace("ixpciecat_v", "", 1)
        ver = int_(ver)
        if ver > chose_ver:
            chose_ver = ver
            chose_file = FileName
    global up_ixe_ver2
    global up_ixe_file2

    up_ixe_ver2 = chose_ver
    up_ixe_file2 = chose_file


def GetIxeCurVer():
    ver = ctypes.c_uint16(0)
    ret = elib.ECAT_McGetIxecatVir(DeviceNo, ctypes.byref(ver))
    if ret == 0:
        ver = ver.value
    else:
        ver = 0

    global cur_ixe_ver
    cur_ixe_ver = ver
    return 0


def UpdateIxe(retry_):
    update = 0
    global cur_ixe_ver
    if up_ixe_ver1 > cur_ixe_ver and up_ixe_ver1 == up_ixe_ver2:
        update = 1
        cur_ixe_ver = up_ixe_ver1
    if update == 0:
        return 0

    file_name = "./firmware/" + up_ixe_file1
    file_size = os.path.getsize(file_name)
    ret = -1
    retry__ = retry_
    while retry__:
        retry__ = retry__ - 1
        upload_size = 0
        last_flag = 0
        with open(file_name, mode='rb') as file:
            while 1:
                file_data = file.read(256)
                if len(file_data) == 0:
                    updateProgress(1)
                    file.close()
                    break
                offset = ctypes.c_uint32(upload_size)
                data = (ctypes.c_uint8 * len(file_data)).from_buffer(bytearray(file_data))
                data_size = ctypes.c_uint32(len(file_data))
                if file_size == (upload_size + len(file_data)):
                    last_flag = ctypes.c_uint8(1)
                else:
                    last_flag = ctypes.c_uint8(0)
                ret = elib.ECAT_Upload_Ixecat(DeviceNo, offset, data, data_size, last_flag)
                if ret != 0:
                    file.close()
                    break
                upload_size += len(file_data)
                updateProgress(upload_size / file_size)
        if ret == 0:
            break
        time.sleep(1)
    if ret != 0:
        print("update firmware failed, {}".format(ret))
        return ret

    file_name = "./firmware/" + up_ixe_file2
    file_size = os.path.getsize(file_name)
    ret = -1
    retry__ = retry_
    while retry__:
        retry__ = retry__ - 1
        upload_size = 0
        last_flag = 0
        with open(file_name, mode='rb') as file:
            while 1:
                file_data = file.read(256)
                if len(file_data) == 0:
                    updateProgress(1)
                    file.close()
                    break
                offset = ctypes.c_uint32(upload_size)
                data = (ctypes.c_uint8 * len(file_data)).from_buffer(bytearray(file_data))
                data_size = ctypes.c_uint32(len(file_data))
                if file_size == (upload_size + len(file_data)):
                    last_flag = ctypes.c_uint8(1)
                else:
                    last_flag = ctypes.c_uint8(0)
                ret = elib.ECAT_Upload_Ixpciecat(DeviceNo, offset, data, data_size, last_flag)
                if ret != 0:
                    file.close()
                    break
                upload_size += len(file_data)
                updateProgress(upload_size / file_size)
        if ret == 0:
            break
        time.sleep(1)
    if ret != 0:
        print("update firmware failed, {}".format(ret))
        return ret

    retry__ = retry_
    while retry__:
        retry__ = retry__ - 1
        ret = elib.ECAT_McSetIxecatVir(DeviceNo, cur_ixe_ver)
        if ret == 0:
            break
        time.sleep(1)
    if ret != 0:
        print("update firmware failed, {}".format(ret))
        return -1

    elib.ECAT_CloseDevice(DeviceNo)

    statTime_ = time.time()
    while 1:
        passTime_ = time.time() - statTime_
        ret = elib.ECAT_OpenDevice(DeviceNo)
        if ret == 0:
            updateProgress(1)
            return ret
        updateProgress(passTime_ / 120)
        if passTime_ > 120:
            print("Time out\nPlease power off/on the computer and try again\n")
            return -1
        time.sleep(1)


def GetIghFileVer1():
    found_ = -1
    chose_ver = 0
    chose_sub_ver = 0
    chose_file = ""
    files = os.listdir("./firmware")
    for FileName in files:
        if FileName.find("ec_generic") == -1:
            continue  # next File
        ver = FileName.replace("ec_generic_V", "", 1)
        ver = ver.replace("ec_generic_v", "", 1)
        ver = int_(ver)
        if ver > chose_ver:
            chose_ver = ver
            chose_file = FileName
    global up_igh_ver1
    global up_igh_file1

    up_igh_ver1 = chose_ver
    up_igh_file1 = chose_file


def GetIghFileVer2():
    found_ = -1
    chose_ver = 0
    chose_sub_ver = 0
    chose_file = ""
    files = os.listdir("./firmware")
    for FileName in files:
        if FileName.find("ec_master") == -1:
            continue  # next File
        ver = FileName.replace("ec_master_V", "", 1)
        ver = ver.replace("ec_master_v", "", 1)
        ver = int_(ver)
        if ver > chose_ver:
            chose_ver = ver
            chose_file = FileName
    global up_igh_ver2
    global up_igh_file2

    up_igh_ver2 = chose_ver
    up_igh_file2 = chose_file


def GetIghCurVer():
    ver = ctypes.c_uint16(0)
    ret = elib.ECAT_GetIghVir(DeviceNo, ctypes.byref(ver))
    if ret == 0:
        ver = ver.value
    else:
        ver = 0

    global cur_igh_ver
    cur_igh_ver = ver
    return 0


def UpdateIgh(retry_):
    update = 0
    global cur_igh_ver
    if up_igh_ver1 > cur_igh_ver and up_igh_ver1 == up_igh_ver2:
        update = 1
        cur_igh_ver = up_igh_ver1
    if update == 0:
        return 0

    file_name = "./firmware/" + up_igh_file1
    file_size = os.path.getsize(file_name)
    ret = -1
    retry__ = retry_
    while retry__:
        retry__ = retry__ - 1
        upload_size = 0
        last_flag = 0
        with open(file_name, mode='rb') as file:
            while 1:
                file_data = file.read(256)
                if len(file_data) == 0:
                    updateProgress(1)
                    file.close()
                    break
                offset = ctypes.c_uint32(upload_size)
                data = (ctypes.c_uint8 * len(file_data)).from_buffer(bytearray(file_data))
                data_size = ctypes.c_uint32(len(file_data))
                if file_size == (upload_size + len(file_data)):
                    last_flag = ctypes.c_uint8(1)
                else:
                    last_flag = ctypes.c_uint8(0)
                ret = elib.ECAT_UploadEc_Generic(DeviceNo, offset, data, data_size, last_flag)
                if ret != 0:
                    file.close()
                    break
                upload_size += len(file_data)
                updateProgress(upload_size / file_size)
        if ret == 0:
            break
        time.sleep(1)
    if ret != 0:
        print("update firmware failed, {}".format(ret))
        return ret

    file_name = "./firmware/" + up_igh_file2
    file_size = os.path.getsize(file_name)
    ret = -1
    retry__ = retry_
    while retry__:
        retry__ = retry__ - 1
        upload_size = 0
        last_flag = 0
        with open(file_name, mode='rb') as file:
            while 1:
                file_data = file.read(256)
                if len(file_data) == 0:
                    updateProgress(1)
                    file.close()
                    break
                offset = ctypes.c_uint32(upload_size)
                data = (ctypes.c_uint8 * len(file_data)).from_buffer(bytearray(file_data))
                data_size = ctypes.c_uint32(len(file_data))
                if file_size == (upload_size + len(file_data)):
                    last_flag = ctypes.c_uint8(1)
                else:
                    last_flag = ctypes.c_uint8(0)
                ret = elib.ECAT_UploadEc_Master(DeviceNo, offset, data, data_size, last_flag)
                if ret != 0:
                    file.close()
                    break
                upload_size += len(file_data)
                updateProgress(upload_size / file_size)
        if ret == 0:
            break
        time.sleep(1)
    if ret != 0:
        print("update firmware failed, {}".format(ret))
        return ret

    elib.ECAT_CloseDevice(DeviceNo)

    statTime_ = time.time()
    while 1:
        passTime_ = time.time() - statTime_
        ret = elib.ECAT_OpenDevice(DeviceNo)
        if ret == 0:
            retry__ = retry_
            while retry__:
                retry__ = retry__ - 1
                ret = elib.ECAT_SetIghVir(DeviceNo, cur_igh_ver)
                if ret == 0:
                    break
                time.sleep(1)
            if ret != 0:
                print("update firmware failed, {}".format(ret))
            else:
                updateProgress(1)
            return ret
        updateProgress(passTime_ / 15)
        if passTime_ > 15:
            print("Time out\nPlease power off/on the computer and try again\n")
            return -1
        time.sleep(1)


if __name__ == '__main__':
    elib = IcpdasEcatLib()
    while 1:
        if InitDevice() != 0:
            break

        GetFpgaFileVer()
        if GetFpgaCurVer() == 0:
            if UpdateFpga(5) != 0:
                break

        GetDaemonFileVer()
        if GetDaemonCurVer() != 0:
            break
        if UpdateDaemon(5) != 0:
            break

        GetIghFileVer1()
        GetIghFileVer2()
        if GetIghCurVer() != 0:
            break
        if UpdateIgh(5) != 0:
            break

        GetIxeFileVer1()
        GetIxeFileVer2()
        if GetIxeCurVer() != 0:
            break
        if UpdateIxe(5) != 0:
            break
        Mesg = "Now is latest version\n" \
               "Firmware Ver:\t{}\n" \
               "Ixe driver Ver:\t{}\n" \
               "ECAT driver Ver:\t{}\n" \
               "Fpga driver Ver:\t{}".format(cur_daemon_ver, cur_ixe_ver, cur_igh_ver, cur_fpga_ver)

        print(Mesg)
        break

    if os.name == 'nt':
        os.system('pause')

